<?php

namespace InSegment\WorkersFramework\Console;

use Shavshukov\EventAppBase\Interfaces\StorageInterface;
use Shavshukov\EventAppBase\Interfaces\ModelInterface;
use Shavshukov\EventAppBase\FileReaders\FileReaderInterface;
use Shavshukov\EventAppBase\Storage\FileStorage;
use Shavshukov\EventAppBase\ApplicationInterface;
use Shavshukov\EventAppBase\Model\YamlModel;

use InSegment\WorkersFramework\Models\Config;

class ReadConfig
{
    /**
     * @param \Shavshukov\EventAppBase\ApplicationInterface $app
     * @param string $configFileName
     * @return string
     */
    public function getConfigPathFromApplication(ApplicationInterface $app, string $configFileName): string
    {
        return implode(DIRECTORY_SEPARATOR, [$app->getAppPath(), $configFileName]);
    }
    
    /**
     * @param \Shavshukov\EventAppBase\FileReaders\FileReaderInterface $fileReader
     * @param string $configFilePath
     * @return \Shavshukov\EventAppBase\Interfaces\StorageInterface
     */
    public function getFileStorage(FileReaderInterface $fileReader, string $configFilePath): StorageInterface
    {
        return new FileStorage($fileReader, $configFilePath);
    }
    
    /**
     * @param \Shavshukov\EventAppBase\Interfaces\StorageInterface $storage
     * @return \Shavshukov\EventAppBase\Interfaces\ModelInterface
     */
    public function getModelFromStorage(StorageInterface $storage): ModelInterface
    {
        $storage->load();
        return YamlModel::extend($storage);
    }
    
    /**
     * @param \Shavshukov\EventAppBase\ApplicationInterface $app
     * @param string|\Shavshukov\EventAppBase\Interfaces\ModelInterface $configSource
     * @return \InSegment\WorkersFramework\Models\Config
     * @throws \Exception
     */
    public function readConfigFromConfigSource(ApplicationInterface $app, $configSource): Config
    {
        if (is_string($configSource)) {
            $configFilePath = $this->getConfigPathFromApplication($app, $configSource);
            $storage = $this->getFileStorage($app->getFileReader(), $configFilePath);
            $model = $this->getModelFromStorage($storage);
            $model->load();
        } else if (!$configSource instanceof StorageInterface) {
            throw new \Exception('Config source must be either filename or a StorageInterface instance');
        } else {
            $model = $configSource;
        }
        
        $config = Config::extend($model);
        return $config;
    }
    
}
