<?php

namespace InSegment\WorkersFramework\Drivers\RabbitMq;

use Shavshukov\EventAppBase\Service\ErrorReport;

use Bunny\Client;
use Bunny\Channel;
use Bunny\Message;

class SyncChannel extends ChannelDriver
{
    
    /**
     * Create instance of RabbitMq service driver connected by async channel
     * 
     * @param \InSegment\WorkersFramework\Drivers\RabbitMq\Connector $connector
     * @param \Bunny\Channel $channel
     * @return \InSegment\WorkersFramework\Drivers\RabbitMq\SyncChannel
     */
    public static function create(Connector $connector, Channel $channel)
    {
        $channel->qos(
            $connector->getPrefetchSize(),
            $connector->getPrefetchCount()    
        );
        
        return new static($channel);
    }

    /**
     * Subscribe to queue and return consumer tag
     * 
     * @param string $queueName
     * @param \InSegment\WorkersFramework\Interfaces\ConsumerInterface $receiver
     * @return string
     */
    public function subscribe(string $queueName, ConsumerInterface $receiver)
    {
        $response = $this->channel->consume(
            function (Message $message, Channel $channel, Client $client) use ($receiver, $queueName) {
                try {
                    $messageWrapped = new QueueMessage($message, $channel, $queueName);
                    $receiver->receiveMessage($messageWrapped, $this);
                } catch (\Throwable $e) {
                    ErrorReport::report($e, false);
                }
            },
            $queueName
        );
            
        return $response->consumerTag;
    }
    
}
